@@ -55,10 +55,25 @@ class AssetSourceType(Enum):
 DATA_QUALITY = 'DATA_QUALITY'
 RECONCILIATION = 'RECONCILIATION'
 
-# SDK_MIN_VERSION_SUPPORT should have the compatible version number as per torch. So even if SDK version number changes
-# but the compatibility with the torch doesn't break then SDK_MIN_VERSION_SUPPORT need not change.
+# MIN_TORCH_BACKEND_VERSION_SUPPORTED is the minimum torch backend version supported by SDK.
 
-SDK_MIN_VERSION_SUPPORT = '2.2.0'
+MIN_TORCH_BACKEND_VERSION_SUPPORTED = '2.2.0'
+
+# minimum torch backend version supported for the api /api/rules/{identifier}
+MIN_TORCH_BACKEND_VERSION_FOR_RULE_ID_API = '2.12.0'
+
+
+@dataclass
+class TorchBuildVersion:
+    buildVersion = None
+    buildDate = None
+
+    def __init__(self, buildVersion=None, buildDate=None, *args, **kwargs):
+        self.buildVersion = buildVersion
+        self.buildDate = buildDate
+
+    def __repr__(self):
+        return f"TorchBuildVersion({self.__dict__})"
 
 
 @dataclass
