@@ -75,39 +75,6 @@ class DatasourceSourceType:
     def __repr__(self):
         return f"DatasourceSourceType({self.__dict__})"
 
-
-class ChildType:
-
-    def __init__(self, canProfile, canSample, id, name, *args, **kwargs):
-        self.canProfile = canProfile
-        self.canSample = canSample
-        self.id = id
-        self.name = name
-
-    def __eq__(self, other):
-        return self.id == other.id
-
-    def __repr__(self):
-        return f"ChildType({self.__dict__})"
-
-
-class RootAsset:
-
-    def __init__(self, assetId=None, alias=None, isCustom=False, childType=None, name=None, parentId=None, *args, **kwargs):
-        self.assetId = assetId
-        self.alias = alias
-        self.isCustom = isCustom
-        if isinstance(childType, dict):
-            self.datasource = ChildType(**childType)
-        else:
-            self.datasource = childType
-        self.name = name
-        self.parentId = parentId
-
-    def __repr__(self):
-        return f"DataSourceRootAsset({self.__dict__})"
-
-
 class PropertyType(Enum):
     INT = auto()
     FLOAT = auto()
