@@ -4,7 +4,7 @@ from dataclasses import dataclass
 from typing import List
 from acceldata_sdk.models.rule import ShortSegment, Label, BackingAsset, RuleThresholdLevel, RuleTag, PolicyGroup, RuleResource, RuleExecution
 
-from acceldata_sdk.models.ruleExecutionResult import ExecutorConfig
+from acceldata_sdk.models.ruleExecutionResult import ExecutorConfig, SparkResourceConfig
 from acceldata_sdk.constants import FailureStrategy, PolicyType
 
 
@@ -36,7 +36,7 @@ class DataQualityRuleExecutionResult(RuleExecutionResult):
 
 
 class DataQualityRuleItemResult(RuleItemResult):
-    def __init__(self, id, ruleItemId, threshold, weightage, isWarning, resultPercent=None, businessItemId=None,
+    def __init__(self, id, ruleItemId, threshold, weightage, isWarning=None, resultPercent=None, businessItemId=None,
                  success=None, error=None, *args, **kwargs):
         super().__init__(id, ruleItemId, threshold, weightage, isWarning, resultPercent, success, error)
         self.businessItemId = businessItemId
@@ -194,7 +194,7 @@ class DataQualityRule:
                  notificationChannels=None, backingAssets=None, createdAt=None, updatedAt=None, thresholdLevel=None,
                  archived=None, archivalReason=None, tenantId=None, createdBy=None, lastUpdatedBy=None, tags=None,
                  segments=None, version=None, executorConfig=None, labels=None, isProtectedResource=None,
-                 policyGroups=None, id=None, *args, **kwargs):
+                 policyGroups=None, id=None, sparkResourceConfig=None, *args, **kwargs):
         self.id = id
         self.name = name
         self.description = description
@@ -241,6 +241,11 @@ class DataQualityRule:
             self.executorConfig = ExecutorConfig(**executorConfig)
         else:
             self.executorConfig = executorConfig
+        if isinstance(sparkResourceConfig, dict):
+            self.sparkResourceConfig = SparkResourceConfig(**sparkResourceConfig)
+        else:
+            self.sparkResourceConfig = sparkResourceConfig
+
         self.labels = list()
         if labels is not None:
             for obj in labels:
