@@ -5,7 +5,12 @@ from typing import List
 from acceldata_sdk.models.job import CreateJob
 from acceldata_sdk.models.span import Span
 from acceldata_sdk.models.span_context import SpanContext
+from acceldata_sdk.datetime_utils import DateTimeFormatter
 from datetime import datetime
+import logging
+
+logger = logging.getLogger('acceldata_sdk_code')
+logger.setLevel(logging.INFO)
 
 
 @dataclass
@@ -44,7 +49,8 @@ class CreatePipeline:
     # updatedAt = None
 
     def __init__(self, uid: str, name: str,
-                 description: str = None, meta: PipelineMetadata = None, **context):
+                 description: str = None, meta: PipelineMetadata = None, createdAt: datetime = None,
+                 updatedAt=None, **context):
         """
         Description:
             Class used for create pipeline in torch catalog
@@ -52,6 +58,8 @@ class CreatePipeline:
         :param name: name of the pipeline
         :param description: (Optional)description of the pipeline
         :param meta: (PipelineMetadata) meta data of the pipeline
+        :param createdAt: (Optional) creation time of the pipeline. If not set, the current time will be used
+        :param updatedAt: (Optional) update time of the pipeline. If not set, the current time will be used
         :param context: context map of the pipeline
         """
         self.uid = uid
@@ -62,6 +70,9 @@ class CreatePipeline:
             self.meta = PipelineMetadata(owner=meta.owner, team=meta.team, codeLocation=meta.codeLocation)
         else:
             self.meta = None
+        self.createdAt = DateTimeFormatter.format_datetime(createdAt)
+        self.updatedAt = DateTimeFormatter.format_datetime(updatedAt)
+
 
     def __eq__(self, other):
         return self.uid == other.uid
@@ -127,8 +138,8 @@ class NodeStatus(Enum):
 
 class PipelineRun:
 
-    def __init__(self, pipelineId, versionId=None, status=PipelineRunStatus.STARTED, id=None, startedAt=None,
-                 finishedAt=None, result=None, client=None, args=None,
+    def __init__(self, pipelineId, versionId=None, status=PipelineRunStatus.STARTED, id=None, startedAt: datetime = None,
+                 finishedAt: datetime = None, result=None, client=None, args=None,
                  continuationId=None, successEvents=None, errorEvents=None, warningEvents=None, avgExecutionTime=None,
                  *argumentss, **kwargs):
         """
@@ -138,8 +149,8 @@ class PipelineRun:
         :param versionId:  pipeline run current version
         :param status: status of the pipeline run
         :param id: pipeline run id
-        :param startedAt: starting time
-        :param finishedAt: finish time
+        :param startedAt: An optional parameter used to define the starting time of the pipeline run. If this parameter is not provided, the pipeline will commence at the time of its creation by default.
+        :param finishedAt: An optional parameter used to specify the completion time of the pipeline run. If this parameter is not provided, it will be automatically set to the time when the pipeline run is finished.
         :param args: additional args for pipeline run
         :param result: pipeline run result
         """
@@ -158,10 +169,11 @@ class PipelineRun:
             self.warningEvents = warningEvents
         if avgExecutionTime is not None:
             self.avgExecutionTime = avgExecutionTime
+        self.startedAt = startedAt
+        self.finishedAt = finishedAt
+
         if id is not None:
             self.id = id
-            self.startedAt = startedAt
-            self.finishedAt = finishedAt
             self.client = client
 
     def __eq__(self, other):
@@ -203,31 +215,28 @@ class PipelineRun:
             raise Exception('To create a job job uid and name are required')
         payload = self._convert_job_to_dict(job)
         res = self.client.create_job(payload, self.pipelineId)
-        if job.bounded_by_span and job.span_uid is not None:
-            associated_job_uids = [job.uid]
-            span_uid_temp = job.span_uid
-            # Get root span to create child span under
-            parent_span_context = self.get_root_span()
-            if parent_span_context is not None:
-                # Create child span under root span
-                span_context = parent_span_context.create_child_span(
-                    uid=span_uid_temp,
-                    context_data={
-                        'time': str(datetime.now())
-                    },
-                    associatedJobUids=associated_job_uids
-                )
+        if job.bounded_by_span:
+            if job.span_uid is not None:
+                associated_job_uids = [job.uid]
+                span_uid_temp = job.span_uid
+                # Get root span to create child span under
+                parent_span_context = self.get_root_span()
+                if parent_span_context is not None:
+                    # Create child span under root span
+                    span_context = parent_span_context.create_child_span(
+                        uid=span_uid_temp,
+                        associatedJobUids=associated_job_uids,
+                        with_explicit_time=job.with_explicit_time
+                    )
+                else:
+                    # Create root span under pipeline_run as root span doesn't exist
+                    self.create_span(
+                        uid=span_uid_temp,
+                        associatedJobUids=associated_job_uids,
+                        with_explicit_time=job.with_explicit_time
+                    )
             else:
-                # Create root span under pipeline_run as root span doesn't exist
-                self.create_span(
-                    uid=span_uid_temp,
-                    context_data={
-                        'time': str(datetime.now())
-                    },
-                    associatedJobUids=associated_job_uids
-                )
-        else:
-            raise Exception('To create a span span_uid is required')
+                raise Exception('To create a span span_uid is required')
         return res
 
     # convert pipeline run to dict type
@@ -250,18 +259,22 @@ class PipelineRun:
     # update run for a pipeline
     def update_pipeline_run(self, result: PipelineRunResult,
                             status: PipelineRunStatus = PipelineRunStatus.STARTED,
-                            context_data: {} = None):
+                            context_data: {} = None, finishedAt: datetime = None):
         """
         Description:
             used to update an existing pipeline run
         :return: updated pipelineRun class instance
         """
+        if finishedAt is not None:
+            finishedAt = finishedAt.isoformat()
+
         update_pipeline_run = PipelineRun(
             pipelineId=self.pipelineId,
             versionId=self.versionId,
             result=result,
             args=context_data,
-            status=status
+            status=status,
+            finishedAt = finishedAt
         )
         payload = self._convert_pipeline_run_to_dict(update_pipeline_run)
         res = self.client.update_pipeline_run(self.id, payload)
@@ -281,20 +294,24 @@ class PipelineRun:
         return span_payload
 
     # create span for pipeline run
-    def create_span(self, uid: str, context_data: dict = None, associatedJobUids = None ):
+    def create_span(self, uid: str, context_data: dict = None, associatedJobUids = None, createdAt: datetime = None, with_explicit_time: bool = False):
         """
         Description:
             used to create span for any pipeline run
         :param associatedJobUids: list of string (job uids)
         :param context_data
         :param uid: span uid
+        :param with_explicit_time: A boolean flag that determines how the span should be started.
+        - If set to True, you can specify the start time for the span in subsequent span events.
+        - If set to False, the span will be automatically started with the current time at the moment of creation.
         :return: SpanContext of the span
         """
         if uid is None:
             raise Exception('Span uid can not be None. You need to pass span uid to create span')
         create_span = Span(
             uid=uid,
-            pipelineRunId=self.id
+            pipelineRunId=self.id,
+            createdAt=createdAt
         )
         if associatedJobUids is None:
             associatedJobUids = []
@@ -302,8 +319,10 @@ class PipelineRun:
             raise Exception('associatedJobUids should be a list')
 
         payload = self._convert_span_to_dict(create_span, associatedJobUids)
+        logger.debug("create_span method invoked with_explicit_time: ", with_explicit_time)
         res = self.client.create_span(self.id, payload)
-        span_context = SpanContext(client=self.client, span=res, context_data= context_data, new_span = True)
+        span_context = SpanContext(client=self.client, span=res, context_data=context_data, new_span=True,
+                                   with_explicit_time=with_explicit_time)
         return span_context
 
     def get_span(self, span_uid: str):
@@ -544,19 +563,15 @@ class Pipeline:
                     # Create child span under root span
                     span_context = parent_span_context.create_child_span(
                         uid=span_uid_temp,
-                        context_data={
-                            'time': str(datetime.now())
-                        },
-                        associatedJobUids=associated_job_uids
+                        associatedJobUids=associated_job_uids,
+                        with_explicit_time=job.with_explicit_time
                     )
                 else:
                     # Create root span under pipeline_run as root span doesn't exist
                     pipeline_run.create_span(
                         uid=span_uid_temp,
-                        context_data={
-                            'time': str(datetime.now())
-                        },
-                        associatedJobUids=associated_job_uids
+                        associatedJobUids=associated_job_uids,
+                        with_explicit_time=job.with_explicit_time
                     )
             else:
                 raise Exception('To create a span span_uid and pipeline_run_id are required')
@@ -589,7 +604,8 @@ class Pipeline:
         return pipeline_run_payload
 
     # create run for a pipeline
-    def create_pipeline_run(self, context_data: {} = None, continuation_id: str = None) -> PipelineRun:
+    def create_pipeline_run(self, context_data: {} = None, continuation_id: str = None,
+                            startedAt: datetime = None) -> PipelineRun:
         """
         Description:
             used to create a pipeline run
@@ -597,11 +613,20 @@ class Pipeline:
         :param continuation_id: continuation_id of pipeline run. Default value is None
         :return: pipelineRun class instance
         """
-        create_pipeline_run = PipelineRun(
-            pipelineId=self.id,
-            args=context_data,
-            continuationId=continuation_id
-        )
+        if startedAt is not None:
+            startedAt = DateTimeFormatter.format_datetime(startedAt)
+            create_pipeline_run = PipelineRun(
+                pipelineId=self.id,
+                args=context_data,
+                continuationId=continuation_id,
+                startedAt=startedAt
+            )
+        else:
+            create_pipeline_run = PipelineRun(
+                pipelineId=self.id,
+                args=context_data,
+                continuationId=continuation_id
+            )
         payload = self._convert_pipeline_run_to_dict(create_pipeline_run)
         res = self.client.create_pipeline_run(payload)
         return res
