@@ -3,7 +3,7 @@ from dataclasses import dataclass, asdict
 from typing import List
 
 from acceldata_sdk.models.ruleExecutionResult import RuleExecutionResult, RuleItemResult, RuleExecutionSummary, RuleResult
-from acceldata_sdk.models.ruleExecutionResult import ExecutorConfig
+from acceldata_sdk.models.ruleExecutionResult import ExecutorConfig, SparkResourceConfig
 from acceldata_sdk.models.rule import ShortSegment, Label, BackingAsset, RuleThresholdLevel, RuleTag, PolicyGroup, RuleResource, RuleExecution
 from acceldata_sdk.constants import FailureStrategy, PolicyType
 
@@ -14,17 +14,18 @@ class ReconRuleExecutionSummary(RuleExecutionSummary):
                  lastMarker=None, leftLastMarker=None, rightLastMarker=None, executionError=None, finishedAt=None,
                  resetPoint=None, persistencePath=None, resultPersistencePath=None, executorConfig=None,
                  markerConfig=None, leftMarkerConfig=None, rightMarkerConfig=None, dataPersistenceEnabled=None,
-                 *args, **kwargs):
-        super().__init__(ruleId, executionMode, executionStatus, resultStatus, startedAt,executionType, thresholdLevel,
+                 sparkResourceConfig=None, *args, **kwargs):
+        super().__init__(ruleId, executionMode, executionStatus, resultStatus, startedAt, executionType, thresholdLevel,
                          ruleVersion, id, ruleName, ruleType, lastMarker, leftLastMarker, rightLastMarker,
                          executionError, finishedAt, resetPoint, persistencePath, resultPersistencePath, executorConfig,
-                         markerConfig, leftMarkerConfig, rightMarkerConfig, dataPersistenceEnabled, isProtectedResource)
+                         sparkResourceConfig, markerConfig, leftMarkerConfig, rightMarkerConfig, dataPersistenceEnabled,
+                         isProtectedResource)
 
     def __repr__(self):
         return f"ReconRuleExecutionSummary({self.__dict__})"
 
 
-class ReconcillationRuleExecutionResult(RuleExecutionResult):
+class ReconciliationRuleExecutionResult(RuleExecutionResult):
     def __init__(self, status, description=None, successCount=None, failureCount=None, leftRows=None, rightRows=None,
                  qualityScore=None, *args, **kwargs):
         super().__init__(status, description, successCount, failureCount, qualityScore)
@@ -32,7 +33,7 @@ class ReconcillationRuleExecutionResult(RuleExecutionResult):
         self.rightRows = rightRows
 
     def __repr__(self):
-        return f"ReconcillationRuleExecutionResult({self.__dict__})"
+        return f"ReconciliationRuleExecutionResult({self.__dict__})"
 
 
 class MappingOperation(Enum):
@@ -75,7 +76,7 @@ class ColumnMapping:
         return f"ColumnMapping({self.__dict__})"
 
 
-class ReconcillationRuleItemResult(RuleItemResult):
+class ReconciliationRuleItemResult(RuleItemResult):
     def __init__(self, id, ruleItemId, threshold, weightage, isRowMatchMeasure, isWarning, columnMapping=None,
                  resultPercent=None, success=None, error=None, *args, **kwargs):
         super().__init__(id, ruleItemId, threshold, weightage, isWarning, resultPercent, success, error)
@@ -86,30 +87,30 @@ class ReconcillationRuleItemResult(RuleItemResult):
             self.columnMapping = columnMapping
 
     def __repr__(self):
-        return f"ReconcillationRuleItemResult({self.__dict__})"
+        return f"ReconciliationRuleItemResult({self.__dict__})"
 
 
-class ReconcillationExecutionResult(RuleResult):
+class ReconciliationExecutionResult(RuleResult):
     def __init__(self, execution, items, meta=None, result=None, *args, **kwargs):
         if isinstance(execution, dict):
             self.execution = ReconRuleExecutionSummary(**execution)
         else:
             self.execution = execution
         if isinstance(result, dict):
-            self.result = ReconcillationRuleExecutionResult(**result)
+            self.result = ReconciliationRuleExecutionResult(**result)
         else:
             self.result = result
         self.items = list()
         for obj in items:
             if isinstance(obj, dict):
-                self.items.append(ReconcillationRuleItemResult(**obj))
+                self.items.append(ReconciliationRuleItemResult(**obj))
             else:
                 self.items.append(obj)
         self.meta = meta
         self.executionId = self.execution.id
 
     def __repr__(self):
-        return f"ReconcillationExecutionResult({self.__dict__})"
+        return f"ReconciliationExecutionResult({self.__dict__})"
 
 
 class ReconciliationMeasurementType(Enum):
@@ -203,10 +204,13 @@ class NotificationPayload:
 
 
 class ReconciliationRule:
-    def __init__(self, name, description, type, enabled, schedule,scheduled,notificationChannels, leftBackingAsset,
-                 rightBackingAsset,timeSecondsOffset, createdAt, updatedAt, thresholdLevel, archived, archivalReason,
-                 tenantId, createdBy, lastUpdatedBy, tags, version, executorConfig, labels, isProtectedResource,
-                 policyGroups, id=None, *args, **kwargs):
+    def __init__(self, name=None, description=None, type=None, enabled=None, schedule=None, scheduled=None,
+                 notificationChannels=None, leftBackingAsset=None,
+                 rightBackingAsset=None, timeSecondsOffset=None, createdAt=None, updatedAt=None, thresholdLevel=None,
+                 archived=None, archivalReason=None,
+                 tenantId=None, createdBy=None, lastUpdatedBy=None, tags=None, version=None, executorConfig=None,
+                 labels=None, isProtectedResource=None,
+                 policyGroups=None, id=None, sparkResourceConfig=None, *args, **kwargs):
         self.id = id
         self.name = name
         self.description = description
@@ -248,6 +252,12 @@ class ReconciliationRule:
             self.executorConfig = ExecutorConfig(**executorConfig)
         else:
             self.executorConfig = executorConfig
+
+        if isinstance(sparkResourceConfig, dict):
+            self.sparkResourceConfig = SparkResourceConfig(**executorConfig)
+        else:
+            self.sparkResourceConfig = sparkResourceConfig
+
         self.labels = list()
         if labels is not None:
             for obj in labels:
