@@ -2,16 +2,19 @@ import logging
 import time
 from abc import ABC
 from typing import List
+from datetime import datetime, timedelta
+from dateutil.relativedelta import relativedelta
 
 import requests
-
-from acceldata_sdk.models.connection import ConnectionType, AnalyticsPipeline, Connection, ConnectionCheck, ConnectionCheckStatus
+import semantic_version
+from acceldata_sdk.models.connection import ConnectionType, AnalyticsPipeline, Connection, ConnectionCheck, \
+    ConnectionCheckStatus
 from acceldata_sdk.models.profile import Profile, ProfileRequest
 from acceldata_sdk.models.rule import Rule, RuleCancelResponse, RuleExecution, ExecutionResult
 from acceldata_sdk.models.tags import TAG, AssetTag, AssetLabel
 from acceldata_sdk.errors import APIError, TorchSdkException
 from acceldata_sdk.models.assetType import AssetType
-from acceldata_sdk.models.datasource import DataSource,SourceType, CrawlerStatus
+from acceldata_sdk.models.datasource import DataSource, SourceType, CrawlerStatus
 from acceldata_sdk.models.job import Job
 from acceldata_sdk.models.pipeline import CreatePipeline, Pipeline, PipelineRun, PipelineInfo, PipelineDetails
 from acceldata_sdk.models.span import Span, SpanContextEvent, CreateSpanEvent
@@ -19,17 +22,19 @@ from acceldata_sdk.client import TorchClientInterFace
 from acceldata_sdk.models.asset import Asset, Metadata
 from requests_toolbelt import MultipartEncoder
 from acceldata_sdk.models.dqrule import DataQualityExecutionResult, DataQualityRule, DataQualityRuleResource
-from acceldata_sdk.models.reconcillationrule import ReconcillationExecutionResult, ReconciliationRuleResource
-from acceldata_sdk.models.ruleExecutionResult import RuleResult, PolicyFilter, ExecutionPeriod, RuleType, RuleExecutionSummary
+from acceldata_sdk.models.reconcillationrule import ReconciliationExecutionResult, ReconciliationRuleResource
+from acceldata_sdk.models.ruleExecutionResult import RuleResult, PolicyFilter, ExecutionPeriod, RuleType, \
+    RuleExecutionSummary
 from acceldata_sdk.constants import AssetSourceType, SdkSupportedVersions, TorchBuildVersion
 from acceldata_sdk.common import Executor
-from acceldata_sdk.constants import PolicyType, FailureStrategy, RuleExecutionStatus
+from acceldata_sdk.constants import PolicyType, FailureStrategy, RuleExecutionStatus, MIN_TORCH_BACKEND_VERSION_FOR_RULE_ID_API
+from acceldata_sdk.models.rule import RuleResource
 from semantic_version import Version, SimpleSpec
 
-
 from typing import List
 from dataclasses import asdict
 import json
+#import re
 from datetime import datetime, timedelta
 
 _HEADERS = {'User-Agent': 'Acceldata-sdk', 'accessKey': None, 'secretKey': None, 'Content-Type': 'application/json'}
@@ -42,6 +47,20 @@ import urllib3
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
 
+def extract_actual_version(torch_version) -> str:
+    if torch_version.buildVersion:
+        parsed_version = Version(torch_version.buildVersion)
+        actual_version = f'{parsed_version.major}.{parsed_version.minor}.{parsed_version.patch}'
+        return actual_version
+    else:
+        raise TorchSdkException(f'Torch version is not available.')
+
+
+def validate_torch_version(actual_version):
+    if Version(MIN_TORCH_BACKEND_VERSION_FOR_RULE_ID_API) not in SimpleSpec(f'<={actual_version}'):
+        raise TorchSdkException(f'This API is not supported for Torch version {actual_version}.')
+
+
 def check_int(input_str):
     try:
         int(input_str)
@@ -50,6 +69,99 @@ def check_int(input_str):
         return False
 
 
+def calculate_minutes_before_time(minutes):
+    """
+    Calculate the time range starting before the given number of minutes from the current time.
+    :param minutes: Number of minutes to subtract from the current time.
+    :return: A tuple containing two datetime objects -
+             the starting time before the given minutes and
+             the ending time as the current datetime.
+    """
+    now = datetime.now()
+    finished_before_time = now
+    started_after_time = finished_before_time - timedelta(minutes=minutes)
+    return started_after_time, finished_before_time
+
+
+def calculate_hours_before_time(hours):
+    """
+    Calculate the time range starting before the given number of hours from the current time.
+    :param hours: Number of hours to subtract from the current time.
+    :return: A tuple containing two datetime objects -
+             the starting time before the given hours and
+             the ending time as the current datetime.
+    """
+    finished_before_time = datetime.now()
+    started_after_time = finished_before_time - timedelta(hours=hours)
+    return started_after_time,finished_before_time
+
+
+def calculate_days_before_time(days):
+    """
+    Calculate the time range starting before the given number of days from today's midnight.
+    :param days: Number of days to subtract from today's midnight.
+    :return: A tuple containing two datetime objects -
+             the starting time before the given days from the beginning of the current day and
+             the ending time as the beginning of the current day.
+    """
+    now = datetime.now()
+    min_time = datetime.min.time()
+    started_after_time = datetime.combine((now - timedelta(days=days)).date(), min_time)
+    finished_before_time = datetime.combine(now.date(), min_time)
+    return started_after_time,finished_before_time
+
+
+def calculate_months_before_time(months_before):
+    """
+    Calculate the time range starting before the given number of months from the beginning of the current month.
+    :param months_before: Number of months to subtract from the current month.
+    :return: A tuple containing two datetime objects -
+             the starting time before the given months from the beginning of the current month and
+             the ending time as the beginning of the current month.
+    """
+    now = datetime.now()
+    first_day_of_months_before = (now - relativedelta(months=months_before)).replace(day=1, hour=0, minute=0, second=0,
+                                                                       microsecond=0)
+    last_day_of_prev_month = now.replace(day=1, hour=0, minute=0, second=0, microsecond=0)
+    started_after_time = first_day_of_months_before
+    finished_before_time = last_day_of_prev_month
+    return started_after_time,finished_before_time
+
+
+def calculate_time_range(filter):
+    if filter.period is not None:
+        now = datetime.now()
+        finished_before_time = now
+        started_after_time = None  # Initialize to None
+        if filter.period == ExecutionPeriod.Last15minutes:
+            started_after_time, finished_before_time = calculate_minutes_before_time(15)
+        elif filter.period == ExecutionPeriod.Last30minutes:
+            started_after_time, finished_before_time = calculate_minutes_before_time(30)
+        elif filter.period == ExecutionPeriod.Last1hour:
+            started_after_time, finished_before_time = calculate_hours_before_time(1)
+        elif filter.period == ExecutionPeriod.Last3hours:
+            started_after_time, finished_before_time = calculate_hours_before_time(3)
+        elif filter.period == ExecutionPeriod.Last6hours:
+            started_after_time, finished_before_time = calculate_hours_before_time(6)
+        elif filter.period == ExecutionPeriod.Last12hours:
+            started_after_time, finished_before_time = calculate_hours_before_time(12)
+        elif filter.period == ExecutionPeriod.Last24hours:
+            started_after_time, finished_before_time = calculate_hours_before_time(24)
+        elif filter.period == ExecutionPeriod.Today:
+            started_after_time = datetime.combine(datetime.now().date(), datetime.min.time())
+        elif filter.period == ExecutionPeriod.Yesterday:
+            started_after_time, finished_before_time = calculate_days_before_time(1)
+        elif filter.period == ExecutionPeriod.Last7days:
+            started_after_time, finished_before_time = calculate_days_before_time(7)
+        elif filter.period == ExecutionPeriod.Thismonth:
+            started_after_time = datetime(datetime.now().year, datetime.now().month, 1)
+        elif filter.period == ExecutionPeriod.Last1month:
+            started_after_time, finished_before_time = calculate_months_before_time(1)
+        elif filter.period == ExecutionPeriod.Last3month:
+            started_after_time, finished_before_time = calculate_months_before_time(3)
+        return finished_before_time, started_after_time
+
+
 class TorchHttpClient(TorchClientInterFace, ABC):
     access_key = None
     secret_key = None
@@ -264,7 +376,8 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return PipelineRun(**response['run'])
 
     # get pipeline run with particular continuation id or run id
-    def get_pipeline_run(self, pipeline_run_id: int = None, continuation_id: str = None, pipeline_id: str = None) -> PipelineRun:
+    def get_pipeline_run(self, pipeline_run_id: int = None, continuation_id: str = None,
+                         pipeline_id: str = None) -> PipelineRun:
         """
             Description:
                 To get pipeline run with particular id or continuation id of any pipeline
@@ -283,7 +396,9 @@ class TorchHttpClient(TorchClientInterFace, ABC):
             supported_versions = self.get_supported_sdk_versions()
             torch_version: str = self.get_torch_version()
             if torch_version.buildVersion is not None and torch_version.buildVersion != "":
-                actual_version = torch_version.buildVersion[:5]
+                # Parse the semantic version string
+                parsed_version = semantic_version.Version(torch_version.buildVersion)
+                actual_version = f'{parsed_version.major}.{parsed_version.minor}.{parsed_version.patch}'
             else:
                 actual_version = supported_versions.maxVersion
             ver_comparator = SimpleSpec(f'<={actual_version}')
@@ -291,7 +406,10 @@ class TorchHttpClient(TorchClientInterFace, ABC):
                 url = f'{self._pipeline_api_base}/pipelines/{pipeline_id}/runs'
                 payload = {"continuationId": continuation_id}
                 response = self._get(url, params=payload)
-                return_run = response["runs"][0]
+                if response["runs"] is not None and len(response["runs"]) > 0:
+                    return_run = response["runs"][0]
+                else:
+                    raise TorchSdkException('No runs found against the given continuation_id.')
             else:
                 # For torch versions lower than 2.5.0 use old call for continuation_id
                 if continuation_id.isdigit() is False:
@@ -417,6 +535,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         :return: SpanContextEvent class instance
         """
         payload = self.convert_span_event_to_dict(span_event)
+        print(f'payload: {payload}')
         if span_event.spanId is None:
             Exception('To update a pipeline run, id is required.')
         response = self._post(
@@ -621,12 +740,31 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         profile_response = Profile(**response['data'])
         return profile_response
 
-    def sample_data(self, id: int):
-        url = f'{self._catalog_api_base}/assets/{id}/sample'
-        response = self._get(
-            url=url
-        )
-        return response
+    def sample_data(self, asset_id: int):
+        url = f'{self._catalog_api_base}/assets/{asset_id}/sample/async'
+        response = self._post(url=url, as_json=True, payload={})
+
+
+        # This is a file/kafka based asset sampling
+        if response is not None and response.get('requestId') is not None:
+            request_id = response['requestId']
+            sample_data_result = self.get_sample_data_result(request_id)
+
+            while sample_data_result['jobExecutionStatus'] == 'IN_PROGRESS':
+                time.sleep(5)
+                sample_data_result = self.get_sample_data_result(request_id)
+
+            return sample_data_result
+        else:
+            return response
+
+
+    def get_sample_data_result(self, request_id: str):
+            url = f'{self._catalog_api_base}/assets/sample/result/{request_id}'
+            response = self._get(
+                url=url
+            )
+            return response
 
     def get_profile_request_details(self, asset_id: int, req_id: int):
         url = f'{self._catalog_api_base}/assets/{asset_id}/profile/{req_id}'
@@ -796,7 +934,8 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         rule_executor = Executor(rule_type, self)
         return rule_executor.get_execution_status(execution_id)
 
-    def get_rule_execution_result(self, rule_type, execution_id, failure_strategy: FailureStrategy = FailureStrategy.DoNotFail):
+    def get_rule_execution_result(self, rule_type, execution_id,
+                                  failure_strategy: FailureStrategy = FailureStrategy.DoNotFail):
         """
         Description:
             To get result of rule execution
@@ -818,7 +957,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         response = self._post(
             url=f'{self._catalog_api_base}/rules/data-quality/{rule_id}/executions',
             payload=payload_temp
-            )
+        )
         return RuleExecution(**response['execution'])
 
     def execute_reconciliation_rule(self, rule_id, incremental=False, pipeline_run_id=None) -> RuleExecution:
@@ -837,7 +976,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
     def get_dq_rule(self, rule_id) -> DataQualityRuleResource:
         response = self._get(
             url=f'{self._catalog_api_base}/rules/data-quality/{rule_id}'
-            )
+        )
         response['client'] = self
         return DataQualityRuleResource(**response)
 
@@ -848,6 +987,27 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         response['client'] = self
         return ReconciliationRuleResource(**response)
 
+    def get_policy(self, identifier) -> RuleResource:
+        """
+        Description:
+                To get an existing policy from torch catalog
+        :param identifier: uid or id of the policy
+        :return:(Policy) policy class instance
+        """
+
+        torch_version = self.get_torch_version()
+        actual_version = extract_actual_version(torch_version)
+        validate_torch_version(actual_version)
+
+        if Version(MIN_TORCH_BACKEND_VERSION_FOR_RULE_ID_API) in SimpleSpec(f'<={actual_version}'):
+            url = f'{self._catalog_api_base}/rules/{identifier}'
+            response = self._get(url)
+            response['client'] = self
+            return Rule(**response)
+        else:
+            raise TorchSdkException(f'This API is supported starting version: {self.MIN_TORCH_BACKEND_VERSION_FOR_RULE_ID_API}')
+
+
     def get_dq_rule_execution_details(self, execution_id) -> ExecutionResult:
         response = self._get(
             url=f'{self._catalog_api_base}/rules/data-quality/executions/{execution_id}'
@@ -864,7 +1024,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         response = self._get(
             url=f'{self._catalog_api_base}/rules/reconciliation/executions/{execution_id}/result'
         )
-        return ReconcillationExecutionResult(**response)
+        return ReconciliationExecutionResult(**response)
 
     def get_dq_rule_result(self, execution_id) -> RuleResult:
         response = self._get(
@@ -897,7 +1057,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return rule_execution_list
 
     def get_all_rules(self, filter: PolicyFilter, page=0, size=25, withLatestExecution=True,
-                               sortBy='updatedAt:DESC') -> RuleResult:
+                      sortBy='updatedAt:DESC') -> RuleResult:
         params = dict()
         params['page'] = page
         params['size'] = size
@@ -920,37 +1080,10 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         if filter.data_sources is not None:
             params['assemblyIds'] = ",".join([str(i.id) for i in filter.data_sources])
         if filter.period is not None:
-            finished_before_time = datetime.now()
-            if filter.period == ExecutionPeriod.Last15minutes:
-                started_after_time = datetime.now() - timedelta(minutes = 15)
-            elif filter.period == ExecutionPeriod.Last30minutes:
-                started_after_time = datetime.now() - timedelta(minutes = 30)
-            elif filter.period == ExecutionPeriod.Last1hour:
-                started_after_time = datetime.now() - timedelta(hours = 1)
-            elif filter.period == ExecutionPeriod.Last3hours:
-                started_after_time = datetime.now() - timedelta(hours = 3)
-            elif filter.period == ExecutionPeriod.Last6hours:
-                started_after_time = datetime.now() - timedelta(hours = 6)
-            elif filter.period == ExecutionPeriod.Last12hours:
-                started_after_time = datetime.now() - timedelta(hours = 12)
-            elif filter.period == ExecutionPeriod.Last24hours:
-                started_after_time = datetime.now() - timedelta(hours = 24)
-            elif filter.period == ExecutionPeriod.Today:
-                started_after_time = datetime.now() - timedelta(hours = 24)
-            elif filter.period == ExecutionPeriod.Yesterday:
-                started_after_time = datetime.now() - timedelta(days = 2)
-                finished_before_time = datetime.now() - timedelta(days = 1)
-            elif filter.period == ExecutionPeriod.Last7days:
-                started_after_time = datetime.now() - timedelta(days = 7)
-            elif filter.period == ExecutionPeriod.Thismonth:
-                started_after_time = datetime.now() - timedelta(days = 30)
-            elif filter.period == ExecutionPeriod.Last1month:
-                started_after_time = datetime.now() - timedelta(days = 30)
-            elif filter.period == ExecutionPeriod.Last3month:
-                started_after_time = datetime.now() - timedelta(days = 90)
-
+            finished_before_time, started_after_time = calculate_time_range(filter)
+            if started_after_time is not None:
+                params['startedAfter'] = int(round(started_after_time.timestamp() * 1000))
             params['finishedBefore'] = int(round(finished_before_time.timestamp() * 1000))
-            params['startedAfter'] = int(round(started_after_time.timestamp() * 1000))
 
         response = self._get(
             url=f'{self._catalog_api_base}/rules',
