@@ -4,18 +4,24 @@ import pprint
 import acceldata_sdk.constants as const
 from acceldata_sdk.models.ruleExecutionResult import RuleType, PolicyFilter, ExecutionPeriod
 
-
 pp = pprint.PrettyPrinter(indent=4)
 
 
 class TestDQPolicy:
     torch_client = TorchClient(**test_const.torch_credentials)
-    dq_policy_id = 1
+
+    dq_policy_id = test_const.dq_policy_id
 
     def test_get_policy(self):
         dq_rule = self.torch_client.get_policy(const.PolicyType.DATA_QUALITY, self.dq_policy_id)
         assert dq_rule is not None
 
+    def test_get_policy_without_type(self):
+        dq_rule = self.torch_client.get_policy(identifier=self.dq_policy_id)
+        pp.pprint("dq_rule")
+        pp.pprint(dq_rule)
+        assert dq_rule is not None
+
     def test_get_all_policy(self):
         filter = PolicyFilter(policyType=RuleType.DATA_QUALITY, enable=True)
         dq_rules = self.torch_client.list_all_policies(filter=filter)
@@ -50,12 +56,18 @@ class TestDQPolicy:
 
 class TestReconPolicy:
     torch_client = TorchClient(**test_const.torch_credentials)
-    recon_policy_id = 3
+    recon_policy_id = test_const.recon_policy_id
 
     def test_get_policy(self):
         recon_rule = self.torch_client.get_policy(const.PolicyType.RECONCILIATION, self.recon_policy_id)
         assert recon_rule is not None
 
+    def test_get_policy_without_type(self):
+        recon_rule = self.torch_client.get_policy(identifier=self.recon_policy_id)
+        pp.pprint("recon_rule")
+        pp.pprint(recon_rule)
+        assert recon_rule is not None
+
     def test_get_all_policy(self):
         filter = PolicyFilter(policyType=RuleType.RECONCILIATION, enable=True)
         recon_rules = self.torch_client.list_all_policies(filter=filter)
